
;*********************************************************************
;  Write LCD-Command Chip 1...  Daten in TEMP_A
;
;  Scratch-Reg: ---
;*********************************************************************

lcd_control:
		out		LCD_DATA,TEMP_A

		cbi		LCD_CTRL,LCD_CS
 	    cbi		LCD_CTRL,LCD_RW
		sbi		LCD_CTRL,LCD_RS

		rcall	lcd_e_clock

        sbi		LCD_CTRL,LCD_CS
        sbi		LCD_CTRL,LCD_RW

		ret

;*********************************************************************
;  Write LCD-Daten Chip 1...  Daten in TEMP_A
;
;  Scratch-Reg: ---
;*********************************************************************

lcd_daten:
		push	TEMP_B
		out		LCD_DATA,TEMP_A

 	    cbi		LCD_CTRL,LCD_RW
		cbi		LCD_CTRL,LCD_RS
		cbi		LCD_CTRL,LCD_CS

		rcall	lcd_e_clock

		clr		TEMP_A
		out		LCD_DDR,TEMP_A			; auf Eingang
		sbi		LCD_CTRL,LCD_RS			; Display auf Instruction
        sbi		LCD_CTRL,LCD_RW			; Display auf Lesen
		
lcd_daten_loop:
		rcall	lcd_e_clock
		sbrc	TEMP_B,LCD_BUSY
		rjmp	lcd_daten_loop

		sbi		LCD_CTRL,LCD_CS
		ser		TEMP_A
		out		LCD_DDR,TEMP_A			; auf Ausgang

		pop		TEMP_B
		ret

;*********************************************************************
;  LCD-E-Clock
;
;  Scratch-Reg: TEMP_A
;*********************************************************************

lcd_e_clock:
		push	TEMP_A
        sbi     LCD_CTRL,LCD_E          ; E auf 1
		ldi		TEMP_A,(CYCLES_PER_US)/3

lcd_e_clock_loop_1:
		dec		TEMP_A
		brne	lcd_e_clock_loop_1

		in		TEMP_B,LCD_DATA_IN
		cbi     LCD_CTRL,LCD_E          ; E auf 0	
		ldi		TEMP_A,(CYCLES_PER_US)/3	; rund 1000ns
		
lcd_e_clock_loop_2:
		dec		TEMP_A
		brne	lcd_e_clock_loop_2

		pop		TEMP_A
		ret

;*********************************************************************
;  LCD-Init
;
;  Scratch-Reg: TEMP_A
;*********************************************************************

lcd_init:
		cbi		LCD_CTRL,LCD_RES
		rcall 	wait_5ms

		sbi		LCD_CTRL,LCD_RES
		rcall 	wait_5ms
			   	
		ldi 	TEMP_A, LCD_MODE		;Set Cursor, Mode, Master, On, Blinken 
		rcall 	lcd_control
		
		ldi 	TEMP_A, (1<<LCD_MODE_MS) | (1<<LCD_MODE_ON)	;Cursor Off, Text, Char intern 
		rcall 	lcd_daten

		ldi 	TEMP_A, LCD_CHAR_PITCH	;Char Pitch H V 
		rcall 	lcd_control
		
		ldi 	TEMP_A, LCD_FONT_6x8	;Char 8x6 
		rcall 	lcd_daten

		ldi 	TEMP_A, LCD_CHAR_ANZ	;Anzahl Zeichen horizontal 
		rcall 	lcd_control
		
		ldi 	TEMP_A, CHAR_PER_LINE-1 
		rcall 	lcd_daten

		ldi 	TEMP_A, LCD_DUTY		;Anzahl Multiplex-Spalten 
		rcall 	lcd_control
		
		ldi 	TEMP_A, LCD_DUTY_WERT
		rcall 	lcd_daten

		ldi 	TEMP_A, LCD_DISP_L		;Ram Startadresse Low
		rcall 	lcd_control
		
		ldi 	TEMP_A, 0x00
		rcall 	lcd_daten

		ldi 	TEMP_A, LCD_DISP_H		;Ram Startadresse High
		rcall 	lcd_control
		
		ldi 	TEMP_A, 0x00
		rcall 	lcd_daten

		ldi 	TEMP_A, LCD_CUR_L		;Cursoradresse Low
		rcall 	lcd_control
		
		ldi 	TEMP_A, 0x00
		rcall 	lcd_daten

		ldi 	TEMP_A, LCD_CUR_H		;Cursoradresse High
		rcall 	lcd_control
		
		ldi 	TEMP_A, 0x00
		rcall 	lcd_daten

		ldi		TEMP_C,LCD_LINES
		clr		TEMP_A

lcd_init_clear_loop:
		push	TEMP_A
		rcall	lcd_clear_line
		pop		TEMP_A
		inc		TEMP_A
		dec		TEMP_C
		brne	lcd_init_clear_loop

		ret


;*********************************************************************
;  Lscht eine Zeile im Display-Ram, Nummer der Zeile in TEMP_A
;
;  Scratch-Reg: TEMP_A, TEMP_B
;*********************************************************************

lcd_clear_line:
		ldi		TEMP_B,CHAR_PER_LINE
		mul		TEMP_A,TEMP_B	
		ldi		TEMP_A,LCD_CUR_L
		rcall	lcd_control
		mov		TEMP_A,MUL_L
		rcall	lcd_daten
		ldi		TEMP_A,LCD_CUR_H
		rcall	lcd_control
		mov		TEMP_A,MUL_H
		rcall	lcd_daten
		ldi		TEMP_A,LCD_WRITE
		rcall	lcd_control
		ldi		TEMP_B,CHAR_PER_LINE

lcd_clear_loop:
		ldi		TEMP_A,' '
		rcall	lcd_daten
        dec		TEMP_B
        brne	lcd_clear_loop

		ret

;*********************************************************************
;  Schreibt eine Zeile als Text 8x6 Font in Display-Ram 
;  In:  Nummer der Zeile in TEMP_A
;  Out: 
;  Scratch-Reg: TEMP_A, TEMP_B, YH, YL
;*********************************************************************

lcd_write_line:
		ldi		TEMP_B,CHAR_PER_LINE
		mul		TEMP_A,TEMP_B	
		ldi		TEMP_A,LCD_CUR_L
		rcall	lcd_control
		mov		TEMP_A,MUL_L
		rcall	lcd_daten
		ldi		TEMP_A,LCD_CUR_H
		rcall	lcd_control
		mov		TEMP_A,MUL_H
		rcall	lcd_daten

		ldi		TEMP_A,LCD_WRITE				; Display Daten schreiben
		rcall	lcd_control

		load_p	Y,LCD_LINE						; auszugebender Text
		ldi		TEMP_B,CHAR_PER_LINE			; Anzahl Zeichen

lcd_write_line_loop:		
		ld		TEMP_A,Y+						; erstes ASCII-Zeichen holen
		rcall	lcd_daten

		dec		TEMP_B							; alle Zeichen?
		brne	lcd_write_line_loop				; nein, weiter

		ret

;*********************************************************************
;  Schreibt Balken in eine Zeile 
;  In:  Nummer der Zeile in TEMP_A, Anzahl Spalten in TEMP_B, Startposition in TEMP_C, Bitmuster in TEMP_D
;  Out: 
;  Scratch-Reg: TEMP_A, TEMP_B, TEMP_C
;*********************************************************************

lcd_write_balken:

/*
		rcall	lcd_set_adresse
		mov		TEMP_A,TEMP_D			; Bitmuster nach TEMP_A
								
lcd_write_balken_loop:
		sbrs	TEMP_C,6				; Chip 2
		rcall	lcd_daten_1
		sbrc	TEMP_C,6				; Chip 1
		rcall	lcd_daten_2
		inc		TEMP_C					; Spaltenzhler
        dec		TEMP_B					; Anzahl Spalten
        brne	lcd_write_balken_loop	; Zeichen fertig?
*/
		ret

;*********************************************************************
;  Setzt Position im Display-Ram, Zeile in TEMP_A, Startadresse in TEMP_C
;
;  Scratch-Reg: TEMP_A, TEMP_C
;*********************************************************************

lcd_set_adresse:

/*
		ori		TEMP_A,$B8
		rcall	lcd_control_1			; Zeile setzen Chip 1
		rcall	lcd_control_2			; Zeile setzen Chip 2
		ldi		TEMP_A,$40
		rcall	lcd_control_1			; Startspalte Chip 1 Position 4			
		rcall	lcd_control_2			; Startspalte Chip 2 Position 0
		
		or		TEMP_A,TEMP_C			; gewnschte Startspalte nach TEMP_A
		sbrs	TEMP_C,6				; mehr als 63 -> Chip 2
		rcall	lcd_control_1			; Startspalte Chip 1 setzen			
		sbrc	TEMP_C,6				; weniger als 64 -> Chip 1
		rcall	lcd_control_2			; Startspalte Chip 2 setzen			
*/
		ret

;*********************************************************************
;  Schreibt eine Zeile als Text 16x12 Font in Display-Ram
;  In:  Nummer der Zeile in TEMP_A, Anzahl Zeichen in TEMP_B, Startposition in TEMP_C
;  Out: 
;  Scratch-Reg: TEMP_A, TEMP_B, TEMP_C, TEMP_D, ZH, ZL, YH, YL, T-FLAG
;*********************************************************************

lcd_write_line_big:
		rjmp	lcd_write_line
/*
		clt								; T = 0 -> erster Durchlauf

lcd_write_line_big_loop:
		push	TEMP_A
		push	TEMP_B
		push	TEMP_C

		rcall	lcd_set_adresse
				
		load_p	Y,LCD_LINE				; auszugebender Text

lcd_write_line_big_loop_1:		
		ld		TEMP_A,Y+				; erstes ASCII-Zeichen holen
		ldi		TEMP_D,CHAR_WIDTH_BIG*2	; Zeichenbreite oben + unten
		mul		TEMP_A,TEMP_D			; Offset zum Tabellenanfang
		load_p	Z,(2*char_tab_big)		; Start Tabelle Big Font
		add		ZL,MUL_L
		adc		ZH,MUL_H

		brts	lcd_write_line_big_loop_1_no_add
		adiw	Z,1						; im 2. Durchlauf erste Bytes des Zeichens

lcd_write_line_big_loop_1_no_add:
		ldi		TEMP_D,CHAR_WIDTH_BIG	; Zeichenbreite

lcd_write_line_big_char_loop_1:
		lpm		TEMP_A,Z+				; Bitmuster holen
		adiw	Z,1						; nchstes Byte (unten) berspringen
		sbrs	TEMP_C,6				; Chip 2
		rcall	lcd_daten_1
		sbrc	TEMP_C,6				; Chip 1
		rcall	lcd_daten_2
		inc		TEMP_C					; Spaltenzhler
        dec		TEMP_D
        brne	lcd_write_line_big_char_loop_1	; Zeichen fertig?

		dec		TEMP_B					; alle Zeichen?
		brne	lcd_write_line_big_loop_1	; nein, weiter

		pop		TEMP_C
		pop		TEMP_B
		pop		TEMP_A

		brts	lcd_write_line_big_end					; zweiter Durchlauf, T = 1 ?

		inc		TEMP_A					; untere Zeile
		set								; T=1 -> 2. Durchlauf
		rjmp	lcd_write_line_big_loop

lcd_write_line_big_end:
*/
		ret

