
;*************************** IRQ - Routinen **************************

;*********************************************************************
;  IRQ Display - 10ms Timer
;  maximale Zyklus-Anzahl:  ohne Subroutinen -> s
;*********************************************************************

irq_timer:
		push	TEMP_A
		in		TEMP_A,SREG						; Statusregister retten
		push	TEMP_A
		push	TEMP_B

		rcall	set_status						; Tuner Feldstrke, Stereo, RDS

		lds		TEMP_A,IRQ_COUNT
		inc		TEMP_A							; bis COUNT_SEK zhlen fr 1s
		sts		IRQ_COUNT,TEMP_A										

		cpi		TEMP_A,IRQ_COUNT/2		
		brne	irq_timer_no_status

		sbr		TUNER_FLAGS,(1<<RDS_SCROLL)

irq_timer_no_status:
		cpi		TEMP_A,COUNT_SEK				; Sekunde rum ?
		brne	irq_timer_key					; nein, nur Tasten abfragen		

		clr		TEMP_A
		sts		IRQ_COUNT,TEMP_A				; Zhler auf 0
		sbr		TUNER_FLAGS,(1<<RDS_SCROLL)		; Text-Scroll

irq_timer_key:
		tst		KEY_CODE						; Taste schon bearbeitet ?
		brne	irq_timer_end					; nein, also nichts machen
	
		in		TEMP_A,KEY_PIN					; Tasten holen
		com		TEMP_A							; umdrehen wegen Low-aktiv
		andi	TEMP_A,KEY_MASK					; Bits ausmaskieren
		cp		TEMP_A,KEY_TEMP					; anderer Status ?
		breq	irq_timer_key_ok				; nein

		mov		KEY_TEMP,TEMP_A					; merken
		rjmp	irq_timer_end					; und Ende

irq_timer_key_ok:
		tst		TEMP_A							; losgelassen ?
		brne	irq_timer_key_press				; nein

		cbr		KEY_FLAGS,(1<<KEY_PRESS)		; Flag lschen
		cbr		KEY_FLAGS,(1<<KEY_REPEAT_OK) 
		clr		KEY_REPEAT	
		rjmp	irq_timer_end					; und Ende
		
irq_timer_key_press:
		sbrs	KEY_FLAGS,KEY_PRESS				; Tasten war losgelassen ?
		rjmp	irq_timer_key_true				; Ja, gltig

		sbrs	KEY_FLAGS,KEY_REPEAT_ON			; soll wiederholt werden?
		rjmp	irq_timer_end					; nein
		
		andi	TEMP_A,REPEAT_MASK				; Taste zum wiederholen ?
		brne	irq_timer_end					; nein
				
		inc		KEY_REPEAT
		cpi		KEY_REPEAT,REPEAT_WAIT			; erstes Warten
		sbrc	KEY_FLAGS,KEY_REPEAT_OK			; noch 1. Mal ?
		cpi		KEY_REPEAT,REPEAT_LOOP			; nein, schon Loop
		brne	irq_timer_end					

		sbr		KEY_FLAGS,(1<<KEY_REPEAT_OK)

irq_timer_key_true:
		clr		KEY_REPEAT	
		sbr		KEY_FLAGS,(1<<KEY_PRESS)
		mov		KEY_CODE,KEY_TEMP				; Taste gltig erkannt

irq_timer_end:
		pop		TEMP_B
		pop		TEMP_A
		out		SREG,TEMP_A
		pop		TEMP_A	
		reti

;*********************************************************************
;  Setzt Stereo und Feldstrke
;  In:  
;  Out: 
;  Out: 
;*********************************************************************
set_status:		
		sbrs	TUNER_FLAGS,ADC_IGNORE				; Dummy-Messung?
		rjmp	set_status_feldstaerke				; nein, gltig

		cbr		TUNER_FLAGS,(1<<ADC_IGNORE)			; Flag lschen, nchste Messung starten
		rjmp	set_status_end

set_status_feldstaerke:
		sbrc	TUNER_FLAGS,RDS_FLAG				; Feldstrke ?
		rjmp	set_status_rds						; nein, RDS

		in		TEMP_B,ADCH
		sts		FELDSTAERKE,TEMP_B					; High-Byte in FELDSTAERKE merken

		ldi		TEMP_B,(1<<REFS0)+(1<<ADLAR)+ADC1 	; RDS Messung vorbereiten
		out		ADMUX,TEMP_B						; VRef = Ub, Pin1 ausgewhlt

		sbr		TUNER_FLAGS,(1<<ADC_IGNORE)			; erste Messung ignorieren weil ungltiger Wert nach VRef-Wechsel
		sbr		TUNER_FLAGS,(1<<RDS_FLAG)			; RDS soll gemessen werden

		rjmp	set_status_end

set_status_rds:
		cbr		TUNER_FLAGS,(1<<RDS_SIGNAL)			; erstmal kein RDS setzen
		in		TEMP_B,ADCH
		cpi		TEMP_B,RDS_REFERENZ
		brlo	set_status_rds_ok

		sbr		TUNER_FLAGS,(1<<RDS_SIGNAL)			; doch, RDS liegt an

set_status_rds_ok:
		ldi		TEMP_B,(1<<REFS0)+(1<<REFS1)+(1<<ADLAR)+ADC0 	; auf Feldstrkemessung umschalten
		out		ADMUX,TEMP_B						; VRef = Interne REF, Pin0 ausgewhlt

		sbr		TUNER_FLAGS,(1<<ADC_IGNORE)			; erste Messung ignorieren weil ungltiger Wert nach VRef-Wechsel
		cbr		TUNER_FLAGS,(1<<RDS_FLAG)			; Feldstrke soll gemessen werden
		
set_status_end:
		sbi		ADCSRA,ADSC							; AD-Wandler starten

		ret


