
;*************************************************************************
;interupt routine receive 1 bit
;*************************************************************************
irq_rds:	
		push	TEMP_A
		in		TEMP_A,SREG
		push	TEMP_A
		push	TEMP_B
		push	TEMP_C
		push	ZH
		push	ZL

		clc									; Carry auf 0, dann
		sbic 	RDS_D_PIN,RDS_DATA			; RDS_DATA lesen
		sec									; wenn 1 dann Carry setzen
		rol 	DATA3						; und vom LSB aus in Data schieben
		rol 	DATA2						;
		rol	 	DATA1						;
		rol 	DATA0						;
		sbrs 	TUNER_FLAGS,RDS_SYNC		; Sync ok
		rjmp	irq_rds_cal_crc				; Prfsumme berechnen, ob syncron			

		dec 	BITCNT						; sonst auf nchstes Bit warten 
		breq	irq_rds_calculate			; alle Bits da						
		rjmp	irq_rds_end

;*************************************************************************
;Syndrom berechnen
;*************************************************************************

irq_rds_calculate:
		ldi		TEMP_A,26					; Alles da
		mov		BITCNT,TEMP_A				; Bitcounter wieder setzen
irq_rds_cal_crc:
		ldi		TEMP_A,0x03					; Prfsummenberechnung	
		and 	DATA0,TEMP_A
		mov 	SYNC_H,DATA0				; CRC merken
		mov 	SYNC_L,DATA1

		load_p	Z,(2*rds_check)				; Tabelle Prfsummen
		ldi		TEMP_C,16					; 16 Bit zu schieben

irq_rds_cal_crc_loop:
		lpm		TEMP_B,Z+					; Low holen Prfbyte
		lpm		TEMP_A,Z+					; High holen
		ror		DATA2
		ror		DATA3						
		brcc	irq_rds_loop_bit_0

		eor 	SYNC_H,TEMP_A				; wenn Bit = 1 Prfbyte verarbeiten
		eor 	SYNC_L,TEMP_B

irq_rds_loop_bit_0:
		dec		TEMP_C						; Bitzhler
		brne	irq_rds_cal_crc_loop

		ror		DATA2						; letztes Carry wieder rein
		ror		DATA3						

;*************************************************************************
;Check welches Syndrom A,B,C,C'
;*************************************************************************
	
		cpi  	RDS_BLOCK,3					; Kontrollwort passend zum Block
		breq 	irq_rds_check_d				; Block 4

		cpi  	RDS_BLOCK,2		
		breq 	irq_rds_check_c				; Block 3

		cpi		RDS_BLOCK,1		
		breq 	irq_rds_check_b				; Block 2

		cpi 	SYNC_L,0xd8					; Block 1
		brne 	irq_rds_not_valid

		cpi 	SYNC_H,0x03					; $03D8
		brne 	irq_rds_not_valid			; ungltig

		rcall 	rds_rolldat						; schiebt die Daten passend
		sts 	BLOCK1H,DATA1				; Block 1 sichern
		sts 	BLOCK1L,DATA2
		sbr 	TUNER_FLAGS,(1<<RDS_SYNC)	; Sync ok setzen
		inc 	RDS_BLOCK					; Block 1 gltig setzen
		rjmp	irq_rds_end						

irq_rds_check_b:
		cpi 	SYNC_L,0xd4					; Block 2
		brne 	irq_rds_not_valid
		
		cpi 	SYNC_H,0x03					; $03D4
		brne 	irq_rds_not_valid			; ungltig
		
		rcall 	rds_rolldat						; schiebt die Daten passend
		sts 	BLOCK2H,DATA1				; Block 2 sichern
		sts 	BLOCK2L,DATA2
		sbr 	TUNER_FLAGS,(1<<RDS_SYNC)	; Sync ok setzen
		inc 	RDS_BLOCK					; Block 1 gltig setzen
		rjmp	irq_rds_end						

irq_rds_check_c:
		lds 	TEMP_B,BLOCK2H				; Block 3
		sbrc 	TEMP_B,3					; Test, ob Block 3 Typ A or B
		rjmp 	irq_rds_check_c2

		cpi 	SYNC_L,0x5c					; Typ A ?
		brne 	irq_rds_not_valid

		cpi 	SYNC_H,0x02					; $025C
		brne 	irq_rds_not_valid			; ungltig

		rcall 	rds_rolldat						; schiebt die Daten passend
		sts 	BLOCK3H,DATA1				; Block 3 Typ A sichern
		sts 	BLOCK3L,DATA2
		sbr 	TUNER_FLAGS,(1<<RDS_SYNC)	; Sync ok setzen
		inc 	RDS_BLOCK					; Block 3 gltig setzen
		rjmp	irq_rds_end					

irq_rds_check_c2:
		cpi 	SYNC_L,0xcc					; Typ B ?
		brne 	irq_rds_not_valid
		
		cpi 	SYNC_H,0x03					; $03CC
		brne 	irq_rds_not_valid			; ungltig
		
		rcall 	rds_rolldat						; schiebt die Daten passend
		sts 	BLOCK3H,DATA1				; Block 3 Typ B sichern
		sts 	BLOCK3L,DATA2
		sbr 	TUNER_FLAGS,(1<<RDS_SYNC)	; Sync ok setzen
		inc 	RDS_BLOCK					; Block 3 gltig setzen
		rjmp	irq_rds_end					

irq_rds_check_d:
		cpi 	SYNC_L,0x58					; Block 4
		brne 	irq_rds_not_valid
		
		cpi 	SYNC_H,0x02
		brne 	irq_rds_not_valid			; ungltig
		
		rcall 	rds_rolldat
		sts 	BLOCK4H,DATA1				; Block 4 sichern
		sts 	BLOCK4L,DATA2
		sbr 	TUNER_FLAGS,(1<<RDS_SYNC)	; Sync ok setzen
		sbr 	TUNER_FLAGS,(1<<RDS_GROUP)	; Gruppe komplett
		rjmp	irq_rds_end						
		
;*************************************************************************
;invalid syndrom handling
;*************************************************************************

irq_rds_not_valid:
		clr 	RDS_BLOCK					; alle Blcke ungltig
		cbr 	TUNER_FLAGS,(1<<RDS_SYNC)	; Sync aus

irq_rds_end:
		pop		ZL
		pop		ZH
		pop		TEMP_C
		pop		TEMP_B
		pop		TEMP_A
		out		SREG,TEMP_A
		pop		TEMP_A
		reti

;*************************************************************************
; organise all bits into 2 bytes
;*************************************************************************
rds_rolldat:
		ror 	DATA0						; um 2 Bit nach rechts schieben
		ror 	DATA1
		ror 	DATA2
		ror 	DATA0
		ror 	DATA1
		ror 	DATA2
		ret
		
