
;*********************************************************************
;  Write LCD-Command Chip 1...  Daten in TEMP_A
;
;  Scratch-Reg: ---
;*********************************************************************

lcd_control_1:
		out		LCD_D_PORT,TEMP_A

		cbi		LCD_C_PORT,LCD_DI
        cbi		LCD_C_PORT,LCD_CSA

		rcall	lcd_e_clock

        sbi		LCD_C_PORT,LCD_DI
        sbi		LCD_C_PORT,LCD_CSA

		ret

;*********************************************************************
;  Write LCD-Command Chip 2...  Daten in TEMP_A
;
;  Scratch-Reg: ---
;*********************************************************************

lcd_control_2:
		out		LCD_D_PORT,TEMP_A

		cbi		LCD_C_PORT,LCD_DI
        cbi		LCD_C_PORT,LCD_CSB

		rcall	lcd_e_clock

        sbi		LCD_C_PORT,LCD_DI
        sbi		LCD_C_PORT,LCD_CSB

		ret

;*********************************************************************
;  Write LCD-Daten Chip 1...  Daten in TEMP_A
;
;  Scratch-Reg: ---
;*********************************************************************

lcd_daten_1:
		out		LCD_D_PORT,TEMP_A

        cbi		LCD_C_PORT,LCD_CSA

		rcall	lcd_e_clock

        sbi		LCD_C_PORT,LCD_CSA

		ret

;*********************************************************************
;  Write LCD-Daten Chip 2...  Daten in TEMP_A
;
;  Scratch-Reg: ---
;*********************************************************************

lcd_daten_2:
		out		LCD_D_PORT,TEMP_A

        cbi		LCD_C_PORT,LCD_CSB

		rcall	lcd_e_clock

        sbi		LCD_C_PORT,LCD_CSB

		ret

;*********************************************************************
;  LCD-E-Clock
;
;  Scratch-Reg: TEMP_A
;*********************************************************************

lcd_e_clock:
		push	TEMP_A
        sbi     LCD_C_PORT,LCD_E          ; E auf 1
		ldi		TEMP_A,(CYCLES_PER_US/3)+1

lcd_e_clock_loop_1:
		dec		TEMP_A
		brne	lcd_e_clock_loop_1

		cbi     LCD_C_PORT,LCD_E          ; E auf 0
		ldi		TEMP_A,(CYCLES_PER_US*5)/3	; rund 5000ns

lcd_e_clock_loop_2:
		dec		TEMP_A
		brne	lcd_e_clock_loop_2

		pop		TEMP_A
		ret

;*********************************************************************
;  LCD-Init
;
;  Scratch-Reg: TEMP_A
;*********************************************************************

lcd_init:
		rcall wait_5ms
			   	
		ldi TEMP_A, 0xC0			;Start Display Line = 0
		rcall lcd_control_1
		rcall lcd_control_2
		
		ldi		TEMP_A,0			; 1. Page

lcd_init_loop_2:
		push	TEMP_A	
		rcall	lcd_clear_line
		pop		TEMP_A
		inc		TEMP_A
		cpi		TEMP_A,8			; fertig?
		brne	lcd_init_loop_2

		ldi TEMP_A, 0x3F			;Display ein
		rcall lcd_control_1
		rcall lcd_control_2
				
		ret

;*********************************************************************
;  Schreibt eine Zeile als Text 8x6 Font in Display-Ram 
;  In:  Nummer der Zeile in TEMP_A, Anzahl Zeichen in TEMP_B, Startposition in TEMP_C
;  Out: 
;  Scratch-Reg: TEMP_A, TEMP_B, TEMP_C, TEMP_D, ZH, ZL, YH, YL
;*********************************************************************

lcd_write_line:
		rcall	lcd_set_adresse
				
		load_p	Y,LCD_LINE				; auszugebender Text

lcd_write_line_loop_1:		
		ld		TEMP_A,Y+				; erstes ASCII-Zeichen holen
		subi	TEMP_A,$20				; Sonder-Codes weg
		ldi		TEMP_D,CHAR_WIDTH		; Zeichenbreite
		mul		TEMP_A,TEMP_D			; Offset zum Tabellenanfang
		load_p	Z,(2*char_tab_6)		; Start Tabelle 6x8 Font
		add		ZL,MUL_L
		adc		ZH,MUL_H

		ldi		TEMP_D,CHAR_WIDTH		; Zeichenbreite

lcd_write_line_char_loop_1:
		lpm		TEMP_A,Z+				; Bitmuster holen
		sbrs	TEMP_C,6				; Chip 2
		rcall	lcd_daten_1
		sbrc	TEMP_C,6				; Chip 1
		rcall	lcd_daten_2
		inc		TEMP_C					; Spaltenzhler
        dec		TEMP_D
        brne	lcd_write_line_char_loop_1	; Zeichen fertig?

		dec		TEMP_B					; alle Zeichen?
		brne	lcd_write_line_loop_1	; nein, weiter

		ret

;*********************************************************************
;  Schreibt eine Zeile als Text 16x12 Font in Display-Ram
;  In:  Nummer der Zeile in TEMP_A, Anzahl Zeichen in TEMP_B, Startposition in TEMP_C
;  Out: 
;  Scratch-Reg: TEMP_A, TEMP_B, TEMP_C, TEMP_D, ZH, ZL, YH, YL, T-FLAG
;*********************************************************************

lcd_write_line_big:
		clt								; T = 0 -> erster Durchlauf

lcd_write_line_big_loop:
		push	TEMP_A
		push	TEMP_B
		push	TEMP_C

		rcall	lcd_set_adresse
				
		load_p	Y,LCD_LINE				; auszugebender Text

lcd_write_line_big_loop_1:		
		ld		TEMP_A,Y+				; erstes ASCII-Zeichen holen
		ldi		TEMP_D,CHAR_WIDTH_BIG*2	; Zeichenbreite oben + unten
		mul		TEMP_A,TEMP_D			; Offset zum Tabellenanfang
		load_p	Z,(2*char_tab_big)		; Start Tabelle Big Font
		add		ZL,MUL_L
		adc		ZH,MUL_H

		brts	lcd_write_line_big_loop_1_no_add
		adiw	Z,1						; im 2. Durchlauf erste Bytes des Zeichens

lcd_write_line_big_loop_1_no_add:
		ldi		TEMP_D,CHAR_WIDTH_BIG	; Zeichenbreite

lcd_write_line_big_char_loop_1:
		lpm		TEMP_A,Z+				; Bitmuster holen
		adiw	Z,1						; nchstes Byte (unten) berspringen
		sbrs	TEMP_C,6				; Chip 2
		rcall	lcd_daten_1
		sbrc	TEMP_C,6				; Chip 1
		rcall	lcd_daten_2
		inc		TEMP_C					; Spaltenzhler
        dec		TEMP_D
        brne	lcd_write_line_big_char_loop_1	; Zeichen fertig?

		dec		TEMP_B					; alle Zeichen?
		brne	lcd_write_line_big_loop_1	; nein, weiter

		pop		TEMP_C
		pop		TEMP_B
		pop		TEMP_A

		brts	lcd_write_line_big_end					; zweiter Durchlauf, T = 1 ?

		inc		TEMP_A					; untere Zeile
		set								; T=1 -> 2. Durchlauf
		rjmp	lcd_write_line_big_loop

lcd_write_line_big_end:
		ret

;*********************************************************************
;  Lscht eine komplette  
;  In:  Nummer der Zeile in TEMP_A
;  Out: 
;  Scratch-Reg: TEMP_A, TEMP_B, TEMP_C, TEMP_D
;*********************************************************************

lcd_clear_line:
		ldi		TEMP_B,128					; Anzahl Spalten
		clr		TEMP_C						; Startspalte
		clr		TEMP_D						; Bitmuster
		rcall	lcd_write_balken

		ret

;*********************************************************************
;  Schreibt Balken in eine Zeile 
;  In:  Nummer der Zeile in TEMP_A, Anzahl Spalten in TEMP_B, Startposition in TEMP_C, Bitmuster in TEMP_D
;  Out: 
;  Scratch-Reg: TEMP_A, TEMP_B, TEMP_C
;*********************************************************************

lcd_write_balken:
		rcall	lcd_set_adresse
		mov		TEMP_A,TEMP_D			; Bitmuster nach TEMP_A
								
lcd_write_balken_loop:
		sbrs	TEMP_C,6				; Chip 2
		rcall	lcd_daten_1
		sbrc	TEMP_C,6				; Chip 1
		rcall	lcd_daten_2
		inc		TEMP_C					; Spaltenzhler
        dec		TEMP_B					; Anzahl Spalten
        brne	lcd_write_balken_loop	; Zeichen fertig?

		ret

;*********************************************************************
;  Setzt Position im Display-Ram, Zeile in TEMP_A, Startadresse in TEMP_C
;
;  Scratch-Reg: TEMP_A, TEMP_C
;*********************************************************************

lcd_set_adresse:
		ori		TEMP_A,$B8
		rcall	lcd_control_1			; Zeile setzen Chip 1
		rcall	lcd_control_2			; Zeile setzen Chip 2
		ldi		TEMP_A,$40
		rcall	lcd_control_1			; Startspalte Chip 1 Position 4			
		rcall	lcd_control_2			; Startspalte Chip 2 Position 0
		
		or		TEMP_A,TEMP_C			; gewnschte Startspalte nach TEMP_A
		sbrs	TEMP_C,6				; mehr als 63 -> Chip 2
		rcall	lcd_control_1			; Startspalte Chip 1 setzen			
		sbrc	TEMP_C,6				; weniger als 64 -> Chip 1
		rcall	lcd_control_2			; Startspalte Chip 2 setzen			

		ret

