;*********************************************************************
;  Initialisiert TDA8425)
;
;  Scratch-Reg: TEMP_A
;*********************************************************************
tda8425_init:

		ldi		TEMP_A,0b11110110
		sts		TDA_BASS,TEMP_A

		ldi		TEMP_A,0b11110110
		sts		TDA_TREBLE,TEMP_A

		ldi		TEMP_A,0b11101111
		sts		TDA_VOL_L,TEMP_A

		ldi		TEMP_A,0b11101111
		sts		TDA_VOL_R,TEMP_A

		ldi		TEMP_A,SWITCH_STEREO_2
		sts		TDA_SWITCH,TEMP_A

		rcall	tda_set_bass
		rcall	tda_set_treble
		rcall	tda_set_vol
		rcall	tda_set_switch

		ret

;*********************************************************************
;  Sendet Kommando an I2C (TDA8425)
;	TEMP_B Wert, TEMP_C TDA-Register  
;
;  Scratch-Reg: TEMP_A
;*********************************************************************

send_i2c:								; Volumen Links
		ldi		TEMP_A,(1<<TWINT)+(1<<TWSTA)+(1<<TWEN)
		rcall	i2c_send_byte
		cpi		TEMP_A,$08				; Start transmitted
		brne	send_i2c_end

		ldi		TEMP_A,TDA8425_WRITE
		out		TWDR,TEMP_A				; Adresse schreiben
		ldi		TEMP_A,(1<<TWINT)+(1<<TWEN)
		rcall	i2c_send_byte
		cpi		TEMP_A,$18				; Adresse gesendet und Ack empfangen

		brne	send_i2c_end

		out		TWDR,TEMP_C				; Sub-Adresse schreiben
		ldi		TEMP_A,(1<<TWINT)+(1<<TWEN)
		rcall	i2c_send_byte
		cpi		TEMP_A,$28				; Sub-Adresse gesendet und Ack empfangen

		brne	send_i2c_end

		out		TWDR,TEMP_B				; Daten senden
		ldi		TEMP_A,(1<<TWINT)+(1<<TWEN)
		rcall	i2c_send_byte
		cpi		TEMP_A,$28				; Daten gesendet und Ack empfangen

		brne	send_i2c_end

send_i2c_end:							; Stop senden
		ldi		TEMP_A,(1<<TWINT)+(1<<TWEN)+(1<<TWSTO)
		out		TWCR,TEMP_A

send_i2c_end_loop:
		in		TEMP_A,TWCR				; warten bis Stop fertig
		sbrc	TEMP_A,TWSTO		
		rjmp	send_i2c_end_loop

		ret

;*********************************************************************
;  Sendet ein Byte per I2C (TDA8425)
;  zu startenden Funktion in TEMP_A, Rckgabe Flags in TEMP_A
;	
;
;  Scratch-Reg: TEMP_A
;*********************************************************************
i2c_send_byte:
		out		TWCR,TEMP_A
		
send_i2c_wait:
		in		TEMP_A,TWCR
		sbrs	TEMP_A,TWINT
		rjmp	send_i2c_wait	; noch nicht fertig			

		in		TEMP_A,TWSR
		andi	TEMP_A,0b11111000	; Flags maskieren

		ret

;*********************************************************************
;  Setzt Volume R und L	 
;
;  Scratch-Reg: TEMP_B, TEMP_C
;*********************************************************************
tda_set_vol:
		ldi		TEMP_C,TDA8425_VOL_L
		lds		TEMP_B,TDA_VOL_L
		rcall	send_i2c

		ldi		TEMP_C,TDA8425_VOL_R
		lds		TEMP_B,TDA_VOL_R
		rjmp	tda_set_end

;*********************************************************************
;  Setzt Bass	 
;
;  Scratch-Reg: TEMP_B, TEMP_C 
;*********************************************************************
tda_set_bass:
		ldi		TEMP_C,TDA8425_BASS
		lds		TEMP_B,TDA_BASS
		rjmp	tda_set_end

;*********************************************************************
;  Setzt Treble	 
;
;  Scratch-Reg: TEMP_B, TEMP_C 
;*********************************************************************
tda_set_treble:
		ldi		TEMP_C,TDA8425_TREBLE
		lds		TEMP_B,TDA_TREBLE
		rjmp	tda_set_end

;*********************************************************************
;  Setzt Switch	 
;
;  Scratch-Reg: TEMP_B, TEMP_C 
;*********************************************************************
tda_set_switch:
		ldi		TEMP_C,TDA8425_SWITCH
		lds		TEMP_B,TDA_SWITCH

tda_set_end:
		rcall	send_i2c

		ret

;*********************************************************************
;  Volumen + 1
;  In:      TDA_VOL_L, TDA_VOL_R
;  Out:     TDA_VOL_L, TDA_VOL_R
;  Scratch: TEMP_A, TEMP_B
;*********************************************************************

volumen_inc:
		lds		TEMP_A,TDA_VOL_L
		ldi		TEMP_B,0b11111100	; 0db

		rcall	audio_inc

		rjmp	volumen_set

;*********************************************************************
;  Volumen - 1
;  In:      TDA_VOL_L, TDA_VOL_R
;  Out:     TDA_VOL_L, TDA_VOL_R
;  Scratch: TEMP_A, TEMP_B
;*********************************************************************

volumen_dec:
		lds		TEMP_A,TDA_VOL_L
		ldi		TEMP_B,0b11011110	; -60db

		rcall	audio_dec

volumen_set:		
		sts		TDA_VOL_L,TEMP_A
		sts		TDA_VOL_R,TEMP_A

		rcall	tda_set_vol
		
		ret

;*********************************************************************
;  Bass + 1
;  In:      TDA_BASS
;  Out:     TDA_BASS
;  Scratch: TEMP_A, TEMP_B 
;*********************************************************************

bass_inc:
		lds		TEMP_A,TDA_BASS
		ldi		TEMP_B,0b11111010	; +12db

		rcall	audio_inc

		rjmp	bass_set

;*********************************************************************
;  Bass - 1
;  In:      TDA_BASS
;  Out:     TDA_BASS
;  Scratch: TEMP_A, TEMP_B 
;*********************************************************************
bass_dec:
		lds		TEMP_A,TDA_BASS
		ldi		TEMP_B,0b11110010	; -12db

		rcall	audio_dec

bass_set:
		sts		TDA_BASS,TEMP_A
		
		rcall	tda_set_bass
		
		ret

;*********************************************************************
;  Treble + 1
;  In:      TDA_TREBLE
;  Out:     TDA_TREBLE
;  Scratch: TEMP_A, TEMP_B 
;*********************************************************************

treble_inc:
		lds		TEMP_A,TDA_TREBLE
		ldi		TEMP_B,0b11111010	; +12db

		rcall	audio_inc

		rjmp	treble_set

;*********************************************************************
;  Treble - 1
;  In:      TDA_TREBLE
;  Out:     TDA_TREBLE
;  Scratch: TEMP_A, TEMP_B 
;*********************************************************************
treble_dec:
		lds		TEMP_A,TDA_TREBLE
		ldi		TEMP_B,0b11110010	; -12db

		rcall	audio_dec

treble_set:
		sts		TDA_TREBLE,TEMP_A
		
		rcall	tda_set_treble
		
		ret


;*********************************************************************
;  Audio - 1
;  In:      TEMP_A Wert, TEMP_B Minimum
;  Out:     TEMP_A neuer Wert
;  Scratch: TEMP_A, TEMP_B 
;*********************************************************************

audio_dec:
		cp		TEMP_A,TEMP_B
		breq	audio_dec_ok		; auf Min

		dec		TEMP_A

audio_dec_ok:
		ret

;*********************************************************************
;  Audio + 1
;  In:      TEMP_A Wert, TEMP_B Minimum
;  Out:     TEMP_A neuer Wert
;  Scratch: TEMP_A, TEMP_B 
;*********************************************************************

audio_inc:
		cp		TEMP_A,TEMP_B
		breq	audio_inc_ok		; auf Max
		
		inc		TEMP_A

audio_inc_ok:
		
		ret
