
; ATmega16

; Secure-Bits fr Mega16:
; 0 gesetzt SUT0 -> Haken in PonyProg
; ext. Quarz 8MHz

; 02.03.2007 M'Soft Berlin

.include "m16def.inc"

.include "Definitionen.inc"

;****************************** Verktor-Liste ****************************

.CSEG
                rjmp    reset

.org INT1addr			;External Interrupt1 Vector Address
                rjmp	irq_rds

.org OC1Aaddr			;Output Compare1A Interrupt Vector Address
                rjmp	irq_timer


;****************************** Programm-Beginn **********************

;*********************************************************************
;  Initialisierungsroutine
;*********************************************************************

; Hardware initialisieren

.org	INT_VECTORS_SIZE

reset:  ldi     TEMP_A,low(RAMEND)				; Stack an das interne Ram-Ende
        out     SPL,TEMP_A
        ldi     TEMP_A,high(RAMEND)
        out     SPH,TEMP_A

; Port A *************************************************************

		ldi		TEMP_A,(1<<RDS_QUALI)+(1<<TUNER_FE)+(1<<TUNER_CE)+(1<<TUNER_MU)
		com		TEMP_A							; kein PullUp, Eingang, ADC	
		out		PORTA,TEMP_A					
		ldi		TEMP_A,(1<<TUNER_CE)+(1<<TUNER_MU)
		out		DDRA,TEMP_A

; Port B *************************************************************

		ser		TEMP_A							; alle Ausgang/PullUp ein
		out     PORTB,TEMP_A                
		out     DDRB,TEMP_A

; Port C *************************************************************

		ldi		TEMP_A,(1<<LCD_E)
		com		TEMP_A							; nue LCD_E auf L
		out     PORTC,TEMP_A 
		ldi		TEMP_A,(1<<LCD_E)+(1<<LCD_RW)+(1<<LCD_RS)+(1<<LCD_CS)+(1<<LCD_RES)	; LCD-Pins auf Ausgang
		out     DDRC,TEMP_A					
	
; Port D *************************************************************
		
		ldi		TEMP_A,(1<<TUNER_CK)							
		com		TEMP_A							; nur TUNER_CK L
		out     PORTD,TEMP_A 
		ldi		TEMP_A,(1<<TUNER_CK)+(1<<TUNER_DI)
		out     DDRD,TEMP_A   

; A/D - Wandler ******************************************************

		ldi		TEMP_A,(1<<REFS0)+(1<<REFS1)+(1<<ADLAR)+ADC0 	; Feldstrke
		out		ADMUX,TEMP_A				; VRef = Interne REF, Pin0 ausgewhlt
		ldi		TEMP_A,(1<<ADEN)+(1<<ADPS2)+(1<<ADPS1)+(1<<ADPS0)
		out		ADCSRA,TEMP_A					; AD-Wandler enable, Prescale 128
		sbr		TUNER_FLAGS,(1<<ADC_IGNORE)

; TIMER 10ms (16Bit) *************************************************

;	Vorteiler 64, Timer 1250

		clr		TEMP_A
		out		TCCR1A,TEMP_A
		ldi		TEMP_A,(1<<CS10)+(1<<CS11)+(1<<WGM12)            ;Prescale 64, CTC-Mode
		out		TCCR1B,TEMP_A
        ldi     TEMP_A,high(TCONST_TIME)		; Timer-Wert
        ldi     TEMP_B,low(TCONST_TIME)			; Timer-Wert
        out     OCR1AH,TEMP_A		   
        out     OCR1AL,TEMP_B		

		ldi		TEMP_A,(1<<OCIE1A)				; Timer-IRQ frei
        out     TIMSK,TEMP_A 

; INT1 L->H **********************************************************

		ldi TEMP_A,(1<<ISC11)+(1<<ISC10)
		out MCUCR,TEMP_A						; Interrupt L->H at INT1
		ldi TEMP_A,(1<<INT1)
		out GICR,TEMP_A							; INT0 ermglichen
 
; I2C	    **********************************************************

		ldi TEMP_A,(1<<TWPS1)					; Vorteier 16
		out TWSR,TEMP_A	
		ldi TEMP_A,I2C_CLOCK
		out TWBR,TEMP_A							; Bitrate setzen
		ldi TEMP_A,(1<<TWIE)	
		out TWCR,TEMP_A							; I2C einschalten

;*********************************************************************				 

; Software initialisieren
		clr		KEY_FLAGS
		clr		KEY_REPEAT
		clr		KEY_TEMP
		clr		KEY_CODE

		clr		MENU_NR
		clr		SUB_MENU_NR

		clr		TUNER_FLAGS
		clr		RDS_BLOCK
		clr		SYNC_L
		clr		SYNC_H
		clr		TEMP_A
		sts		SCROLL_COUNTER,TEMP_A

		sbr		KEY_FLAGS,(1<<KEY_REPEAT_ON)	; Tastenrepeat zulassen
		sbr		KEY_FLAGS,(1<<MENU_RDS)			; RDS wird angezeigt, wenn vorhanden

		rcall	eeprom_read
		lds		TEMP_A,SENDER_0
		cpi		TEMP_A,$FF
		breq	speicher_not_ok

		rcall	calc_checksum					; Senderspeicher testen
		breq	speicher_ok						; ist ok, weiter

speicher_not_ok:
		ldi		TEMP_A,SPEICHER_ANZ
		load_p	Y,F_INIT
		load_p	Z,SENDER_0

speicher_set_default:
		st		Z+,YH
		st		Z+,YL
		adiw	Z,14							; insgesamt 16 Byte weiter
		dec		TEMP_A
		brne	speicher_Set_default

speicher_ok:		
		clr		TEMP_A
		sts		SPEICHER_AKT,TEMP_A

		ldi		TEMP_A,1
		sts		FELDSTAERKE,TEMP_A
								
		rcall	lcd_init						; Display initialisieren
		rcall	pll_init
		rcall	tda8425_init

		rcall	show_menu

		sei
										 
;*********************************************************************
;  Main-Loop
;*********************************************************************

main:					
		tst		KEY_CODE
		brne	main_key
		
		sbic	TUNER_IN,TUNER_OK			; Muting, wenn kein Sender empfangen wird
		sbi		TUNER_OUT_2,TUNER_MU

		sbis	TUNER_IN,TUNER_OK
		cbi		TUNER_OUT_2,TUNER_MU

		rcall	show_tuner_freq_big
		rcall	show_tuner_status

		sbrs	KEY_FLAGS,MENU_RDS			; RDS anzeigen?
		rjmp	main						; nein

		sbrs	TUNER_FLAGS,RDS_SIGNAL		; ja, aber RDS vorhanden?
		rjmp	main						; nein
			
		sbrc 	TUNER_FLAGS,RDS_GROUP		; ja, aber Gruppe komplett?
		rcall	decode_rds					; ja, dekodieren
		
		rcall	show_rds_status				; RDS Status.Block anzeigen
		rcall	show_rds					; und Radiotext anzeigen
		rjmp	main

main_key:
		load_p	Z,(2*menu_1)				; Zeiger auf Mentabelle
		mov		TEMP_A,MENU_NR				; aktuelle Mennummer holen
		sbrs	KEY_CODE,KEY_MENU			; Men-Taste?
		rjmp	main_key_no_menu			; nein

		rcall	save_menu_daten				; ja, alte Daten speichern, wenn ntig			
		sbrc	KEY_FLAGS,MENU_SAVE		
		rjmp	main_key_menu_reset

		mov		TEMP_A,MENU_NR				; aktuelle Mennummer holen
		inc		TEMP_A
		cpi		TEMP_A,MENU_MAX				; letztes Men vorbei?
		brne	main_key_no_max_menu		; nein, ok

main_key_menu_reset:		
		clr		TEMP_A						; sonst auf erstes Men
		cbr		KEY_FLAGS,(1<<MENU_SAVE)		
		 
main_key_no_max_menu:
		mov		MENU_NR,TEMP_A				; neue Mennummer merken			

main_key_no_menu:
		lsl		TEMP_A						; *8 (4 Adressen)
		lsl		TEMP_A
		lsl		TEMP_A
		add		ZL,TEMP_A					; zum Tabellenanfang dazu - Mengruppe
		clr		TEMP_A
		adc		ZH,TEMP_A

		sbrc	KEY_CODE,KEY_MENU			; Mentaste ?
		rjmp	main_key_jmp				; ja, Show Men

		adiw	Z,2							; nchste Adresse ENTER
		sbrc	KEY_CODE,KEY_ENTER			; Enter?
		rjmp	main_key_jmp				; ja

		adiw	Z,2							; nchste Adresse ENTER
		sbrc	KEY_CODE,KEY_DOWN			; Down?
		rjmp	main_key_jmp				; ja

		adiw	Z,2							; nchste Adresse ENTER
		sbrc	KEY_CODE,KEY_UP				; Up?
		rjmp	main_key_jmp				; ja

		rjmp	main_end					; kann eigentlich nicht passieren...

main_key_jmp:
		lpm		XL,Z+						; Zeiger auf Subroutine holen
		lpm		XH,Z
		mov		ZL,XL						; nach Z
		mov		ZH,XH
		icall								; und aufrufen

main_end:
		clr		KEY_CODE					; Taste ist abgearbeitet
		rjmp	main


; --- Tasten - Auswertung -------------------------------------------

.include "IRQ_10ms.inc"

.include "IRQ_RDS.inc"

.include "Menue.inc"

.include "Subroutinen.inc"

.include "EEprom.inc"

.include "NF_TDA8425.inc"

.include "Tuner_PLL.inc"

.include "Tuner_RDS.inc"

.include "HD61830B.inc"

.include "Tabellen.inc"

.include "Font.inc"

.include "eeprom.dat"
