;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;                 LCD-Routinen                ;;
;;                 ============                ;;
;;				               ;;
;; DB0-DB7:       PB0-PB7    LCD_DATEN                  ;;
;; RS:            PA0        LCD_CTRL                  ;;
;; CS1:           PA1                          ;;
;; A0:            PA2                          ;;
;; E:             PA3                          ;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;*********************************************************************
;  Write LCD-Command...  Daten in TEMP_A
;
;  Scratch-Reg: ---
;*********************************************************************

lcd_control:
		cbi LCD_CTRL, LCD_A0
		out LCD_DATEN, TEMP_A
        cbi LCD_CTRL, LCD_CS
		nop
		rcall	lcd_clock			
        sbi LCD_CTRL, LCD_CS

		ret

;*********************************************************************
;  Write LCD-Daten...  Daten in TEMP_A
;
;  Scratch-Reg: ---
;*********************************************************************

lcd_daten:
        sbi LCD_CTRL, LCD_A0
		out LCD_DATEN, TEMP_A
        cbi LCD_CTRL, LCD_CS
		rcall	lcd_clock			
        sbi LCD_CTRL, LCD_CS

		ret

;*********************************************************************
;  LCD-Init
;
;  Scratch-Reg: TEMP_A, ZL, ZH
;*********************************************************************

lcd_init:
     	ldi	TEMP_B,10
		cbi LCD_CTRL, LCD_RW

lcd_init_loop_1:
		rcall delay50us
        dec	TEMP_B

        brne	lcd_init_loop_1
		sbi LCD_CTRL, LCD_RW

		rcall	wait_5ms
				   
		ldi TEMP_A, 0x40			;Start Display Line = 0
		rcall lcd_control
		
		ldi TEMP_A, 0xA0			;ADC = normal (Anzeigerichtung nicht gespiegelt)
		rcall lcd_control
		
		ldi TEMP_A, 0xA3			;LCD-Bias = 1/7 fr 4,5-5,5V    1/9 fr 3,3-4,5V
		rcall lcd_control
		
		ldi TEMP_A, 0xC0			;Output Mode = nicht gespiegelt
		rcall lcd_control
		
		ldi TEMP_A, 0x2F			;LCD-Spannungserzeugung/Regelung einschalten
		rcall lcd_control
		
		ldi TEMP_A, 0x20			;V5 Regulator Resistor Ratio = 0
		rcall lcd_control
		
		ldi TEMP_A, 0xAC			;Static Indicator off
		rcall lcd_control
		
		ldi TEMP_A, 0x81			;Kontrasteinstellung auswhlen
		rcall lcd_control
		
		ldi TEMP_A, 0x32			;Kontrast = mittel (fr Bias 1/7: 20-50, fr 1/9: 2-10)
		rcall lcd_control
		
		;ldi TEMP_A, 0xA5			;Displaytest an
		;rcall lcd_control
		
		ldi TEMP_A, 0xAF			;Display ein
		rcall lcd_control
		
		ldi TEMP_A, 0xA6			;Display normal (nicht invertiert)
		rcall lcd_control
		
		ldi TEMP_A, 0xA4			;Displaytest aus
		rcall lcd_control
		
		rcall lcd_test
		
		ret

;*********************************************************************
;  LCD-Test
;
;  Scratch-Reg: TEMP_A, ZL, ZH
;*********************************************************************
lcd_test:
		   ldi TEMP_A, 0x40			;Display Start Line
		   rcall lcd_control
		   ldi TEMP_A, 0xB0			;Page Address
		   rcall lcd_control
		   ldi TEMP_A, 0x10			;Column Adrress (upper bit)
		   rcall lcd_control
		   ldi TEMP_A, 0x00			;Column Adrress (lower bit)
		   rcall lcd_control
		   ldi TEMP_A, 0xFF
		   ldi TEMP_B, 0x80

lcd_test_loop_1:
		   rcall lcd_daten
           dec	TEMP_B
           brne	lcd_test_loop_1

		   ldi TEMP_A, 0xB1			;Page Address
		   rcall lcd_control
		   ldi TEMP_A, 0x10			;Column Adrress (upper bit)
		   rcall lcd_control
		   ldi TEMP_A, 0x00			;Column Adrress (lower bit)
		   rcall lcd_control
		   ldi TEMP_A, 0x81
		   ldi TEMP_B, 0x3F

lcd_test_loop_2:
		   rcall lcd_daten
           dec	TEMP_B
           brne	lcd_test_loop_2

		   ret

;*********************************************************************
;  Clock LCD E-Impuls
;
;  Scratch-Reg: ---
;*********************************************************************

lcd_clock:
        sbi     LCD_CTRL,LCD_E          ; E auf 1		; 750nS
        nop
		nop
		nop
		nop
		cbi     LCD_CTRL,LCD_E          ; E auf 0

		ret
