
;**************************** Allgemeine Routinen **************************

;***************************************************************************
;  Rom nach Ram kopieren Z - Rom-Adresse, Y- Ram-Adresse, Ende bei 0-Byte
;
;  Scratch-Reg: ZH, ZL, YH, YL, TEMP_A
;***************************************************************************

copy_rom_txt:
        lpm		TEMP_A,Z+
		tst		TEMP_A
		breq	copy_rom_txt_end

        st      Y+,TEMP_A
        brne    copy_rom_txt

copy_rom_txt_end:
        ret

;*********************************************************************
;  Warte 5ms
;
;  Scratch-Reg: -
;*********************************************************************

wait_5ms:
        push    ZL
        push    ZH

        load_p  Z,5000*(CYCLES_PER_US/4)
		        ; 5 mS warten, 4 = Anzahl Befehls-Zyclen vom sbiw + brcc

d_loop:
        sbiw    ZL,1
        brne    d_loop

        pop     ZH
        pop     ZL

        ret

;*********************************************************************
;  Warte 50s
;
;  Scratch-Reg: -
;*********************************************************************
wait_50us:
		ldi  TEMP_A,50*(CYCLES_PER_US/3)

wait_50us_loop:
		dec  TEMP_A
		brne wait_50us_loop

		ret

;***************************************************************************
;  Convertiert TEMP_A in ASCII (high-TEMP_B, low-TEMP_A) Bereich 0...99
;
;  Scratch-Reg: TEMP_A, TEMP_B
;***************************************************************************

convert_ascii:
		clr		TEMP_B						; Zehner-Byte lschen

conv_loop:
		subi	TEMP_A,10					; Wert -10	
		brcs	conv_end					; Ende wenn bertrag

		inc		TEMP_B						; sonst Zehner +1
		rjmp	conv_loop					; und weiter

conv_end:
		subi	TEMP_A,-10					; letzte Subtraktion wieder zurck

		ori		TEMP_A,$30					; nach ASCII wandeln
		ori		TEMP_B,$30		

		ret

;*********************************************************************
; 16Bit bin to BCD-packed
; In:TEMP_A Low In, TEMP_B High_In
; Out: tBCD2 eine Stelle, tBCD1, tBCD0 jeweils 2 Stellen
; Scratch-Reg: ZH, ZL. TEMP_A, TEMP_B, TEMP_C, tBCD2, tBCD1, tBCD0
;*********************************************************************

bin2bcd16:
		ldi		TEMP_C,16					; Bitzhler
		clr		BCD_2						; Ergebnis lschen (3 bytes)
		clr		BCD_1
		clr		BCD_0
		clr		ZH

bin2bcd16_bit_loop:
		lsl		TEMP_A						; Input schieben
		rol		TEMP_B						; durch alle Bytes
		rol		BCD_0				
		rol		BCD_1	
		rol		BCD_2
		dec		TEMP_C						; Bitzhler
		breq	bin2bcd16_end				; fertig ?

		ldi		ZL,AtBCD2+1					; Z auf Ergebnis MSB + 1

bin2bcd16_conv_bcd:
		ld		TEMP_D,-Z					; Z-1, dann Daten nach D holen
		subi	TEMP_D,-$03					; + $03
		sbrc	TEMP_D,3					; wenn Bit 3 nicht 0
		st		Z,TEMP_D					; zurckschreiben

		ld		TEMP_D,Z					; Alten oder neuen Wert wiederholen
		subi	TEMP_D,-$30					; + $30
		sbrc	TEMP_D,7					; wenn Bit 7 nicht 0
		st		Z,TEMP_D					; zurckschreiben
		cpi		ZL,AtBCD0					; alle 3 Bytes bearbeitet?
		brne	bin2bcd16_conv_bcd			; nein, weiter

		rjmp	bin2bcd16_bit_loop			; ja, nchstes Bit 

bin2bcd16_end:
		ret									

;*********************************************************************
;  Berechnet Checksumme fr EEPROM
;  In:      162 Byte ab SENDER_0
;  Out:     Checksumme in TEMP_B, Z gesetzt, wenn ok
;  Scratch: 
;*********************************************************************

calc_checksum:
		load_p	Z,EE_DATA_ANF
		ldi		TEMP_A,(EE_DATA_END-1)-EE_DATA_ANF ; ohne CHECKSUM
		clr		TEMP_B

calc_checksum_loop:
		ld		TEMP_C,Z+
		eor		TEMP_B,TEMP_C
		dec		TEMP_A
		brne	calc_checksum_loop
		
		ld		TEMP_C,Z						; Checksumme holen
		cp		TEMP_C,TEMP_B					; stimmt Checksumme?
		
		ret			
