;***************************************************************************
;  Init PLL
;
;  Scratch-Reg: ZH, ZL, YH, YL
;***************************************************************************

pll_init:
		ldi		TEMP_A,0
		sts		PLL_CTRL,TEMP_A

		ldi		TEMP_A,REF_25+REF_FMIN
		sts		PLL_REF,TEMP_A

		ldi		TEMP_A,0b00001000				; PORT und Counter
		sts		PLL_PORT,TEMP_A

		ldi		TEMP_A,high(F_INIT)
		sts		PLL_DIV_H,TEMP_A

		ldi		TEMP_A,low(F_INIT)
		sts		PLL_DIV_L,TEMP_A
		
		rcall	send_pll_daten
		rcall	show_tuner_freq_big
		rcall	show_tuner_status

		ret

;*********************************************************************
;  Zeigt Frequenz Stereo/Mono und Feldstrke an
;  In:      PLL_DIV_L, PLL_DIV_H, FELDSTAERKE, FLAG_TU_STEREO, RDS_SIGNAL
;  Out:     Werte im Display Zeile 1
;  Scratch: ZH, ZL, YH, YL, TEMP_A, TEMP_B, TEMP_V
;*********************************************************************

show_tuner_status:
		lds		TEMP_B,FELDSTAERKE
		subi	TEMP_B,$10					; Skalierung des Feldstrkewertes auf 1..8
		brcc	show_tuner_status_ok		; Feldstrke ist > 0

		clr		TEMP_B						; sonst auf 0 setzen

show_tuner_status_ok:
		inc		TEMP_B
		cpi		TEMP_B,65					; Lnge
		brlo	show_status_feldstaerke_ok

		ldi		TEMP_B,64					; Lnge

show_status_feldstaerke_ok:
		push	TEMP_B						; Lnge merken

		ldi		TEMP_A,2					; Zeile
		ldi		TEMP_C,4					; Startspalte 
		ldi		TEMP_D,$7E					; Bitmuster			
		rcall	lcd_write_balken

		pop		TEMP_C						; Lnge jetzt Startposition
		subi	TEMP_C,-4					; Startspalte dazu
		ldi		TEMP_B,68					; maximale Lnge + Startposition
		sub		TEMP_B,TEMP_C				; - schon geschriebener Balken
		breq	show_status_stereo			; nichts mehr aufzufllen
		
		ldi		TEMP_A,2					; Zeile
		clr		TEMP_D						; Bitmuster
		rcall	lcd_write_balken				

show_status_stereo:				
		load_p	Y,LCD_LINE
		ldi		TEMP_A,'S'
		sbic	TUNER_IN,TUNER_ST
		ldi		TEMP_A,'M'
		st		Y+,TEMP_A

		ldi		TEMP_A,' '					; Leerzeichen vor RDS
		st		Y+,TEMP_A

		sbrc	TUNER_FLAGS,RDS_SIGNAL		; RDS ?
		rjmp	show_tuner_status_rds		; ja

		st		Y+,TEMP_A					; Leerzeichen
		st		Y+,TEMP_A
		st		Y+,TEMP_A
		rjmp	show_tuner_status_rds_end

show_tuner_status_rds:
		ldi		TEMP_A,'R'
		st		Y+,TEMP_A
		ldi		TEMP_A,'D'
		st		Y+,TEMP_A
		ldi		TEMP_A,'S'
		st		Y+,TEMP_A

show_tuner_status_rds_end:
		ldi		TEMP_A,' '					; Leerzeichen vor Platznummer
		st		Y+,TEMP_A

		st		Y,TEMP_A					; erstmal Leerzeichen					
		mov		TEMP_A,MENU_NR
		cpi		TEMP_A,M_T_SET
		breq	show_tuner_status_platz

		cpi		TEMP_A,M_T_SPEICHER
		brne	show_tuner_status_end

show_tuner_status_platz:
		lds		TEMP_A,SPEICHER_AKT
		ori		TEMP_A,$30					; nach ASCII
		st		Y,TEMP_A

show_tuner_status_end:
		ldi		TEMP_A,2					; Zeile 5
		ldi		TEMP_B,7					; Anzahl
		ldi		TEMP_C,4+(12*CHAR_WIDTH)
		rcall	lcd_write_line

		ret

;*********************************************************************
;  Zeigt Frequenz gro an
;  In:      PLL_DIV_L, PLL_DIV_H, FELDSTAERKE, FLAG_TU_STEREO, RDS_SIGNAL
;  Out:     Werte im Display Zeile 1
;  Scratch: ZH, ZL, YH, YL, TEMP_A, TEMP_B, TEMP_V
;*********************************************************************

show_tuner_freq_big:
		load_p	Y,LCD_LINE
		lds		XL,PLL_DIV_L
		lds		XH,PLL_DIV_H

		mov		TEMP_A,XL
		mov		TEMP_B,XH
		lsl		TEMP_A
		rol		TEMP_B
		lsl		TEMP_A
		rol		TEMP_B			; *4
		add		TEMP_A,XL		; +1* = 5*
		adc		TEMP_B,XH

		subi	TEMP_A,low(F_ZF)
		sbci	TEMP_B,high(F_ZF)

		rcall	bin2bcd16

		mov		TEMP_A,BCD_2
		andi	TEMP_A,$0F
		ori		TEMP_A,'0'
		cpi		TEMP_A,'0'
		brne	show_tuner_freq_big_not_0

		ldi		TEMP_A,' '			;BIG_SPACE

show_tuner_freq_big_not_0:
		st		Y+,TEMP_A

		mov		TEMP_A,BCD_1
		swap	TEMP_A
		andi	TEMP_A,$0F
		ori		TEMP_A,'0'
		st		Y+,TEMP_A

		mov		TEMP_A,BCD_1
		andi	TEMP_A,$0F
		ori		TEMP_A,'0'
		st		Y+,TEMP_A

		ldi		TEMP_A,','			; BIG_KOMMA
		st		Y+,TEMP_A

		mov		TEMP_A,BCD_0
		swap	TEMP_A
		andi	TEMP_A,$0F
		ori		TEMP_A,'0'
		st		Y+,TEMP_A

		mov		TEMP_A,BCD_0
		andi	TEMP_A,$0F
		ori		TEMP_A,'0'
		st		Y+,TEMP_A

		load_p	Z,(2*mhz_txt)
		rcall	copy_rom_txt

		ldi		TEMP_A,0					; Zeile 0
		ldi		TEMP_B,10					; Anzahl Zeichen
		ldi		TEMP_C,4					; Startspalte
		rcall	lcd_write_line_big

		ret

;***************************************************************************
;  Setzt / Lscht Zwangs-Mono (VCO aus)
;
;  Scratch-Reg: ZH, ZL, YH, YL, TEMP_A
;***************************************************************************

set_stereo_mono:				
		sbrs	TUNER_FLAGS,PLL_MONO		; VCO aus?
		rjmp	set_stereo_mono_off			; ausschalten

		cbr		TUNER_FLAGS,(1<<PLL_MONO)
		lds		TEMP_A,PLL_PORT
		cbr		TEMP_A,CTRL_STEREO			; OUT5 auf 1
		sts		PLL_PORT,TEMP_A
		rjmp	set_stereo_mono_ok

set_stereo_mono_off:
		sbr		TUNER_FLAGS,(1<<PLL_MONO)
		lds		TEMP_A,PLL_PORT
		sbr		TEMP_A,CTRL_STEREO			; OUT5 auf 0
		sts		PLL_PORT,TEMP_A

set_stereo_mono_ok:
		rcall	send_pll_daten

		ret

;***************************************************************************
;  Sucht Frequenz abwrts
;
;  Scratch-Reg: ZH, ZL, YH, YL, TEMP_A
;***************************************************************************

frequenz_search_down:				
		rcall	frequenz_down

		rcall	wait_tuning

		sbic	TUNER_IN,TUNER_OK
		rjmp	frequenz_search_down

		ret

;***************************************************************************
;  Sucht Frequenz aufwrts
;
;  Scratch-Reg: ZH, ZL, YH, YL, TEMP_A
;***************************************************************************

frequenz_search_up:				
		rcall	frequenz_up

		rcall	wait_tuning

		sbic	TUNER_IN,TUNER_OK
		rjmp	frequenz_search_up

		ret

;***************************************************************************
;  Add Freq Erhht Frequenz in PLL_DIV_H, PLL_DIV_L um 1
;
;  Scratch-Reg: ZH, ZL, YH, YL, TEMP_A
;***************************************************************************

frequenz_down:
		push	ZH
		push	ZL

		lds		ZL,PLL_DIV_L
		lds		ZH,PLL_DIV_H
		sbiw	Z,1
		cpi		ZH,high(F_MIN-1)		; unten ?
		brne	frequenz_ok

		cpi		ZL,low(F_MIN-1)
		brne	frequenz_ok

		ldi		ZH,high(F_MAX)
		ldi		ZL,low(F_MAX)

		rjmp	frequenz_ok

;***************************************************************************
;  Add Freq Erhht Frequenz in PLL_DIV_H, PLL_DIV_L um 1
;
;  Scratch-Reg: ZH, ZL, YH, YL, TEMP_A
;***************************************************************************

frequenz_up:
		push	ZH
		push	ZL

		sbic	TUNER_IN,TUNER_OK
		sbi		TUNER_OUT_2,TUNER_MU

		sbis	TUNER_IN,TUNER_OK
		cbi		TUNER_OUT_2,TUNER_MU

		lds		ZL,PLL_DIV_L
		lds		ZH,PLL_DIV_H
		adiw	Z,1
		cpi		ZH,high(F_MAX+1)		; oben ?
		brne	frequenz_ok
		cpi		ZL,low(F_MAX+1)
		brne	frequenz_ok

		ldi		ZH,high(F_MIN)
		ldi		ZL,low(F_MIN)

frequenz_ok:
		sts		PLL_DIV_H,ZH
		sts		PLL_DIV_L,ZL	

		rcall	send_pll_daten
		rcall	show_tuner_freq_big
		rcall	show_tuner_status

		sbic	TUNER_IN,TUNER_OK
		sbi		TUNER_OUT_2,TUNER_MU

		sbis	TUNER_IN,TUNER_OK
		cbi		TUNER_OUT_2,TUNER_MU

		rcall	rds_buf_clear

 		pop		ZL
		pop		ZH
        ret

;***************************************************************************
;  Sendet 32 Bit aus PLL_DATA_0...PLL_DATA_3 an die PLL
;
;  Scratch-Reg: ZH, ZL, YH, YL
;***************************************************************************

send_pll_daten:
		sbi		TUNER_OUT_2,TUNER_MU	; stumm schalten
		rcall	wait_5ms
		ldi		TEMP_A,PLL_MODE_1		; alle 32Bit senden
		ldi		TEMP_B,4				; 4 Bit

send_pll_daten_mode:
		cbi		TUNER_OUT_1,TUNER_DI	; 0 raus
		ror		TEMP_A
		brcc	send_pll_daten_mode_clock ; ok

		sbi		TUNER_OUT_1,TUNER_DI	; 1 raus
		
send_pll_daten_mode_clock:
		rcall	pll_ck_wait
		sbi		TUNER_OUT_1,TUNER_CK	; Clock H
		rcall	pll_ck_wait
		cbi		TUNER_OUT_1,TUNER_CK	; Clock L

		dec		TEMP_B
		brne	send_pll_daten_mode		; weiter nchstes Bit

		in		TEMP_A,GICR
		cbr		TEMP_A,(1<<INT1)		; RDS-IRQ sperren, da Register mit RDS geteilt werden
		out		GICR,TEMP_A

		lds		DATA0,PLL_CTRL
		lds		DATA1,PLL_REF
		lds		DATA2,PLL_PORT
		lds		DATA3,PLL_DIV_H
		lds		BITCNT,PLL_DIV_L

		ldi		TEMP_A,36				; Anzahl der Bits
		rcall	pll_ck_wait
		sbi		TUNER_OUT_2,TUNER_CE	; auf H zum Schreiben

send_pll_daten_loop:
		rcall	pll_ck_wait
		cbi		TUNER_OUT_1,TUNER_DI	; 0 raus
		ror		DATA0
		ror		DATA1
		ror		DATA2
		ror		DATA3
		ror		BITCNT	
		brcc	send_pll_daten_clock	; ok

		sbi		TUNER_OUT_1,TUNER_DI	; 1 raus
		
send_pll_daten_clock:
		rcall	pll_ck_wait
		sbi		TUNER_OUT_1,TUNER_CK	; Clock H
		rcall	pll_ck_wait
		cbi		TUNER_OUT_1,TUNER_CK	; Clock L

		dec		TEMP_A
		brne	send_pll_daten_loop		; weiter nchstes Bit

		rcall	pll_ck_wait
		cbi		TUNER_OUT_2,TUNER_CE	; auf L zum bernehmen

		in		TEMP_A,GICR
		sbr		TEMP_A,(1<<INT1)		; RDS-IRQ wieder freigeben
		out		GICR,TEMP_A

		ret

;***************************************************************************
;  Warteschleife fr PLL-Takt
;
;  Scratch-Reg: ZH, ZL, YH, YL
;***************************************************************************

pll_ck_wait:
		ldi		TEMP_C,CYCLES_PER_US
		
pll_ck_wait_loop:
		dec		TEMP_C
		brne	pll_ck_wait_loop
		
		ret

;***************************************************************************
;  Warteschleife fr Suchlauf ca, 50ms
;
;  Scratch-Reg: ZH, ZL, YH, YL
;***************************************************************************

wait_tuning:
		ldi		TEMP_A,10

wait_tuning_loop:
		rcall	wait_5ms
		dec		TEMP_A
		brne	wait_tuning_loop

		ret

