;*********************************************************************
;  Decodiert RDS-Daten
;  
;  Scratch-Reg: -
;*********************************************************************

decode_rds:
		in		TEMP_A,GICR
		cbr		TEMP_A,(1<<INT1)			; RDS-IRQ sperren
		out		GICR,TEMP_A

		load_p	Z,RADIOBUF										
		cbr 	TUNER_FLAGS,(1<<RDS_GROUP)	; Group-Flag lschen
		clr 	RDS_BLOCK					; Block-Counter lschen
		lds 	TEMP_A,BLOCK1H
		lds 	TEMP_B,PI					; letzten PI mit empfangenen PI vergleichen
		cp 		TEMP_A,TEMP_B
		brne 	decode_rds_newpi			; neuer PI

		lds 	TEMP_A,BLOCK1L
		lds 	TEMP_B,PI+1
		cp 		TEMP_A,TEMP_B
		brne 	decode_rds_newpi			; neuer PI

		rjmp 	decode_rds_test_group		; Gruppe testen

decode_rds_newpi:
		lds 	TEMP_A,BLOCK1H
		sts 	PI,TEMP_A					; neuen PI merken
		lds		TEMP_A,BLOCK1L
		sts 	PI+1,TEMP_A

		ldi		XL,low(RADIOTEXT)			; RADIOTEXT lschen
		ldi		XH,high(RADIOTEXT)
		ldi 	TEMP_A,' '
		ldi 	TEMP_B,64+32				; Lnge Buffer, RADIOBUF mit lschen

decode_rds_clear_rt:
		st 		X+, TEMP_A
		dec 	TEMP_B
		brne 	decode_rds_clear_rt

decode_rds_test_group:
		lds 	TEMP_A,BLOCK2H
		andi 	TEMP_A,0b11111000			; Gruppe 0A ?
		breq 	decode_rds_group_0			; ja
		
		cpi 	TEMP_A,0b00001000			; Gruppe 0B ?
		brne 	decode_rds_group_2a			; nein, nchste

;*************************************************************************
;group 0A and 0B, program service name and TA-Flag
;*************************************************************************
decode_rds_group_0:
		lds 	TEMP_A,BLOCK2L
		andi 	TEMP_A,0b00000011				; PS Adresse ausmaskieren
		rol 	TEMP_A							; * 2 Zeichen
		add		ZL,TEMP_A
		clr		TEMP_A
		add		ZH,TEMP_A
		lds 	TEMP_A,BLOCK4H
		st 		Z+,TEMP_A						; show 1st PS char
		lds 	TEMP_A,BLOCK4L
		st 		Z+,TEMP_A						; show 2nd PS char

		load_p	Z,RADIOBUF+9										
		lds 	TEMP_A,BLOCK2L
		sbrc 	TEMP_A,4						; TA flag ?
		rjmp 	decode_rds_ta_set
		
		ldi 	TEMP_A,'-'
		st		Z+,TEMP_A						; "-" wenn kein TA
		rjmp 	decode_rds_end_group	

decode_rds_ta_set:
		ldi 	TEMP_A,'*'
		st		Z+,TEMP_A						; "*" wenn TA
		rjmp 	decode_rds_end_group

;*************************************************************************
;group 2A, Radiotext
;*************************************************************************
decode_rds_group_2a:
		lds 	TEMP_A,BLOCK2H
		andi 	TEMP_A,0b11110000
		cpi 	TEMP_A,0b00100000				; Gruppe 2A ?
		brne 	decode_rds_group_4a				; nein

		lds 	TEMP_A,BLOCK2L
		andi 	TEMP_A,0x0f						; mask char addr
		lsl 	TEMP_A							; one addr for 4 chars
		lsl 	TEMP_A
		mov 	TEMP_B,TEMP_A
		ldi		XL,low(RADIOTEXT)				; store radiotext in SRAM
		ldi		XH,high(RADIOTEXT)
		add 	XL, TEMP_B
		lds 	TEMP_A,BLOCK3H
		st 		X+, TEMP_A
		lds 	TEMP_A,BLOCK3L
		st 		X+, TEMP_A
		lds 	TEMP_A,BLOCK4H
		st 		X+, TEMP_A
		lds 	TEMP_A,BLOCK4L
		st 		X+, TEMP_A
		rjmp 	decode_rds_end_group

;*************************************************************************
;group 4A, clock
;*************************************************************************
decode_rds_group_4a:
		lds 	TEMP_A,BLOCK2H
		andi 	TEMP_A,0b11110000
		cpi 	TEMP_A,0b01000000				; Gruppe 4A ?
		brne 	decode_rds_end_group			; nein, Ende
	
		load_p	Z,RADIOBUF+11										
		lds 	TEMP_C,BLOCK3L					; Stunde
		andi 	TEMP_C,0b00000001				; MSB Stunden Bit
		clc
		lds 	TEMP_B,BLOCK4L					; Temp
		lds 	TEMP_D,BLOCK4H					; Minute
		rol 	TEMP_B
		rol 	TEMP_D
		rol 	TEMP_C	
		rol 	TEMP_B
		rol 	TEMP_D
		rol 	TEMP_C
		
		rol 	TEMP_D
		rol 	TEMP_C
		rol 	TEMP_D
		rol 	TEMP_C							; Stunden im Register

		ror 	TEMP_B
		ror 	TEMP_B							; BLOCK4L im Original
		sbrc 	TEMP_B, 5						; check if we need to add or sub the offset
		rjmp 	decode_rds_sub_offset

		ror		TEMP_B							; make hour steps instead of normal 30min steps
		andi 	TEMP_B, 0b00011111				; mask offset value
		add 	TEMP_C, TEMP_B
		rjmp 	decode_rds_go_ahead

decode_rds_sub_offset:
		ror		TEMP_B							; make hour steps instead of normal 30min steps
		andi 	TEMP_B, 0b00011111	
		sub		TEMP_C, TEMP_B

decode_rds_go_ahead:
		mov 	TEMP_A,TEMP_C
		rcall 	convert_ascii					; Stunden nach BCD
		st		Z+,TEMP_B
		st		Z+,TEMP_A
		ror 	TEMP_D							
		ror 	TEMP_D
		ldi 	TEMP_A,':'
		st		Z+,TEMP_A						
		andi	TEMP_D,0b00111111				; Minuten maskieren
		mov 	TEMP_A,TEMP_D				
		rcall 	convert_ascii					; Minuten nach BCD
		st		Z+,TEMP_B
		st		Z+,TEMP_A

		ldi		TEMP_A,' '
		st		Z+,TEMP_A
		ldi		TEMP_A,'U'
		st		Z+,TEMP_A
		ldi		TEMP_A,'h'
		st		Z+,TEMP_A
		ldi		TEMP_A,'r'
		st		Z+,TEMP_A

decode_rds_end_group:
		in		TEMP_A,GICR
		sbr		TEMP_A,(1<<INT1)				; RDS-IRQ wieder freigeben
		out		GICR,TEMP_A
		ret							

;*********************************************************************
;  Anzeigen der RDS-Status-Buffer
;  
;  Scratch-Reg: -
;*********************************************************************

show_rds_status:
		load_p	Z,RADIOBUF
		load_p	X,LCD_LINE
		ldi		TEMP_A,20

show_rds_status_loop:
		ld		TEMP_B,Z+
		st		X+,TEMP_B
		dec		TEMP_A
		brne	show_rds_status_loop

		ldi		TEMP_A,4
		ldi		TEMP_B,20
		ldi		TEMP_C,4		
		rcall	lcd_write_line		
		ret


;*********************************************************************
;  Zeigt RDS-Daten aus Radiotext
;
;  Scratch-Reg: Y, Z, TEMP_A, TEMP_B
;*********************************************************************

show_rds:
		sbrs	TUNER_FLAGS,RDS_SCROLL
		ret

		load_p	Y,RADIOTEXT
		ldi		TEMP_C,low(RADIOTEXT+64) 	; fr das Ende	
		lds		TEMP_A,SCROLL_COUNTER
		inc		TEMP_A
		cpi		TEMP_A,64					; Bufferende ?
		brne	show_rds_end

		clr		TEMP_A

show_rds_end:
		sts		SCROLL_COUNTER,TEMP_A
		add		YL,TEMP_A
		clr		TEMP_A
		adc		YH,TEMP_A

		load_p	Z,LCD_LINE
		ldi		TEMP_B,21

show_rds_loop:
		ld		TEMP_A,Y+
		st		Z+,TEMP_A
		cp		YL,TEMP_C
		brne	show_rds_buf_end

		load_p	Y,RADIOTEXT

show_rds_buf_end:
		dec		TEMP_B
		brne	show_rds_loop

		ldi		TEMP_A,5					; Zeile 6
		ldi		TEMP_B,20					; Anzahl
		ldi		TEMP_C,4
		rcall	lcd_write_line
		cbr		TUNER_FLAGS,(1<<RDS_SCROLL)
		ret

;***************************************************************************
;  Lscht alle RDS-Buffer
;
;  Scratch-Reg: ZH, ZL, YH, YL, TEMP_A
;***************************************************************************
rds_buf_clear:
		push	ZH
		push	ZL
		push	TEMP_A
		push	TEMP_B

		load_p	Z,RADIOTEXT
		ldi		TEMP_B,96
		ldi		TEMP_A,' '

rds_buf_clear_loop_1:
		st		Z+,TEMP_A
		dec		TEMP_B
		brne	rds_buf_clear_loop_1

		pop		TEMP_B
		pop		TEMP_A
		pop		ZL
		pop		ZH

		ret

